﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows;
using Telerik.Windows.Controls;
using System.Collections.ObjectModel;

namespace SilverlightRadContextMenuCustomTheme
{
	public partial class MainPage : UserControl
	{
		private static IList<Popup> OpenedPopups = new List<Popup>();

		static MainPage()
		{
			EventManager.RegisterClassHandler(typeof(MainPage), RadContextMenu.OpenedEvent, new RoutedEventHandler(ContextMenuOpened), true);
			EventManager.RegisterClassHandler(typeof(MainPage), RadContextMenu.ClosedEvent, new RoutedEventHandler(ContextMenuClosed), true);

			EventManager.RegisterClassHandler(typeof(MainPage), RadMenuItem.SubmenuOpenedEvent, new RadRoutedEventHandler(SubmenuOpened), true);
			EventManager.RegisterClassHandler(typeof(MainPage), RadMenuItem.SubmenuClosedEvent, new RadRoutedEventHandler(SubmenuClosed), true);
		}

		public MainPage()
		{
			InitializeComponent();

			this.MouseMove += OnMouseMove;
		}

		void OnMouseMove(object sender, MouseEventArgs e)
		{
			ObservableCollection<string> items = new ObservableCollection<string>();

			// this code will not work correctly if browser is Zoomed.
			Point globalMousePosition = e.GetPosition(null);
			foreach (UIElement element in GetElementsInScreenCoordinates<Control>(globalMousePosition))
			{
				items.Add(element.ToString());
			}

			lb.ItemsSource = items;
		}

		private Popup GetPopup(Control control)
		{
			if (control != null)
			{
				int childrenCount = VisualTreeHelper.GetChildrenCount(control);
				if (childrenCount > 0)
				{
					FrameworkElement controlRootElement = VisualTreeHelper.GetChild(control, 0) as FrameworkElement;
					if (controlRootElement != null)
					{
						return controlRootElement.FindName("PART_Popup") as Popup;
					}
				}
			}

			return null;
		}

		static void SubmenuOpened(object sender, RadRoutedEventArgs e)
		{
			MainPage page = sender as MainPage;
			if (page != null)
			{
				Popup popup = page.GetPopup(e.OriginalSource as Control);
				if (popup != null)
				{
					OpenedPopups.Add(popup);
					if (popup.Child != null)
					{
						popup.Child.MouseMove += page.OnMouseMove;
					}
				}
			}
		}

		static void SubmenuClosed(object sender, RadRoutedEventArgs e)
		{
			MainPage page = sender as MainPage;
			if (page != null)
			{
				Popup popup = page.GetPopup(e.OriginalSource as Control);
				if (popup != null)
				{
					OpenedPopups.Remove(popup);
					if (popup.Child != null)
					{
						popup.Child.MouseMove -= page.OnMouseMove;
					}
				}
			}
		}

		static void ContextMenuClosed(object sender, RoutedEventArgs e)
		{
			RadContextMenu contextMenu = (e as RadRoutedEventArgs).Source as RadContextMenu;
			MainPage page = sender as MainPage;
			if (page != null && contextMenu != null)
			{
				Popup popup = contextMenu.Parent as Popup;
				if (popup != null)
				{
					OpenedPopups.Remove(popup);
					if (popup.Child != null)
					{
						popup.Child.MouseMove -= page.OnMouseMove;
					}
				}
			}
		}

		static void ContextMenuOpened(object sender, RoutedEventArgs e)
		{
			RadContextMenu contextMenu = (e as RadRoutedEventArgs).Source as RadContextMenu;
			MainPage page = sender as MainPage;
			if (page != null && contextMenu != null)
			{
				Popup popup = contextMenu.Parent as Popup;
				if (popup != null)
				{
					OpenedPopups.Add(popup);
					if (popup.Child != null)
					{
						popup.Child.MouseMove += page.OnMouseMove;
					}
				}
			}
		}

		static IEnumerable<T> GetElementsInScreenCoordinates<T>(Point mousePosition) where T : UIElement
		{
			return GetElementsInScreenCoordinatesFromOpenedPopups<T>(mousePosition).Concat(Application.Current.RootVisual.GetElementsInHostCoordinates<T>(mousePosition)).Distinct<T>();
		}

		static IEnumerable<T> GetElementsInScreenCoordinatesFromOpenedPopups<T>(Point mousePosition) where T : UIElement
		{
			var popups = Enumerable.Empty<Popup>();

			// Here we take RegisteredPopups.Reverse - this mean that we start HitTest from the LAST opened popup.
			// This is done because in SL2-4 you cannot bring popups to front (using ZIndex) so the last registered popup is opened last.
			popups = OpenedPopups.Reverse();

			return popups.SelectMany(p => p.Child.GetElementsInHostCoordinates<T>(mousePosition));
		}
	}

	static class VisualTreeExtensions
	{
		internal static IEnumerable<T> GetElementsInHostCoordinates<T>(this UIElement subtree, Point position) where T : UIElement
		{
			if (subtree == null)
			{
				return Enumerable.Empty<T>();
			}

			return VisualTreeHelper.FindElementsInHostCoordinates(position, subtree).OfType<T>();
		}
	}
}